@@ -5,6 +5,7 @@ import os
 import socket
 import tempfile
 import time
+from dns.resolver import NoAnswer, Answer, Resolver
 
 from eventlet.support import greendns
 from eventlet.support.greendns import dns
@@ -35,20 +36,37 @@ class TestHostsResolver(tests.LimitedTestCase):
         hr = _make_host_resolver()
         hr.hosts.write(b'line0\n')
         hr.hosts.flush()
-        assert hr._readlines() == ['line0']
+        assert list(hr._readlines()) == ['line0']
         hr._last_stat = 0
         hr.hosts.write(b'line1\n')
         hr.hosts.flush()
-        assert hr._readlines() == ['line0', 'line1']
+        assert list(hr._readlines()) == ['line0', 'line1']
+        # Test reading of varied newline styles
         hr._last_stat = 0
-        hr.hosts.write(b'#comment0\nline0\n #comment1\nline1')
-        assert hr._readlines() == ['line0', 'line1']
+        hr.hosts.seek(0)
+        hr.hosts.truncate()
+        hr.hosts.write(b'\naa\r\nbb\r  cc  \n\n\tdd ee')
+        hr.hosts.flush()
+        assert list(hr._readlines()) == ['aa', 'bb', 'cc', 'dd ee']
+        # Test comments, including inline comments
+        hr._last_stat = 0
+        hr.hosts.seek(0)
+        hr.hosts.truncate()
+        hr.hosts.write(b'''\
+# First couple lines
+# are comments.
+line1
+#comment
+line2 # inline comment
+''')
+        hr.hosts.flush()
+        assert list(hr._readlines()) == ['line1', 'line2']
 
     def test_readlines_missing_file(self):
         hr = _make_host_resolver()
         hr.hosts.close()
         hr._last_stat = 0
-        assert hr._readlines() == []
+        assert list(hr._readlines()) == []
 
     def test_load_no_contents(self):
         hr = _make_host_resolver()
@@ -163,6 +181,20 @@ class TestHostsResolver(tests.LimitedTestCase):
         res = set(hr.getaliases('host.example.com'))
         assert res == set(['host'])
 
+    def test_hosts_case_insensitive(self):
+        name = 'example.com'
+        hr = _make_host_resolver()
+        hr.hosts.write(b'1.2.3.4 ExAmPlE.CoM\n')
+        hr.hosts.flush()
+        hr._load()
+
+        ans = hr.query(name)
+        rr = ans.rrset[0]
+        assert isinstance(rr, greendns.dns.rdtypes.IN.A.A)
+        assert rr.rdtype == dns.rdatatype.A
+        assert rr.rdclass == dns.rdataclass.IN
+        assert rr.address == '1.2.3.4'
+
 
 def _make_mock_base_resolver():
     """A mocked base resolver class"""
@@ -173,6 +205,7 @@ def _make_mock_base_resolver():
         aliases = ['cname.example.com']
         raises = None
         rr = RR()
+        rr6 = RR()
 
         def query(self, *args, **kwargs):
             self.args = args
@@ -182,7 +215,10 @@ def _make_mock_base_resolver():
             if hasattr(self, 'rrset'):
                 rrset = self.rrset
             else:
-                rrset = [self.rr]
+                if self.rr6 and self.args[1] == dns.rdatatype.AAAA:
+                    rrset = [self.rr6]
+                else:
+                    rrset = [self.rr]
             return greendns.HostsAnswer('foo', 1, 1, rrset, False)
 
         def getaliases(self, *args, **kwargs):
@@ -191,6 +227,47 @@ def _make_mock_base_resolver():
     return Resolver
 
 
+class TestUdp(tests.LimitedTestCase):
+
+    def setUp(self):
+        # Store this so we can reuse it for each test
+        self.query = greendns.dns.message.Message()
+        self.query.flags = greendns.dns.flags.QR
+        self.query_wire = self.query.to_wire()
+        super(TestUdp, self).setUp()
+
+    def test_udp_ipv6(self):
+        with tests.mock.patch('eventlet.support.greendns.socket.socket.recvfrom',
+                              return_value=(self.query_wire,
+                                            ('::1', 53, 0, 0))):
+            greendns.udp(self.query, '::1')
+
+    def test_udp_ipv6_timeout(self):
+        with tests.mock.patch('eventlet.support.greendns.socket.socket.recvfrom',
+                              side_effect=socket.timeout):
+            with tests.assert_raises(dns.exception.Timeout):
+                greendns.udp(self.query, '::1', timeout=0.1)
+
+    def test_udp_ipv6_addr_zeroes(self):
+        with tests.mock.patch('eventlet.support.greendns.socket.socket.recvfrom',
+                              return_value=(self.query_wire,
+                                            ('0:00:0000::1', 53, 0, 0))):
+            greendns.udp(self.query, '::1')
+
+    def test_udp_ipv6_wrong_addr_ignore(self):
+        with tests.mock.patch('eventlet.support.greendns.socket.socket.recvfrom',
+                              side_effect=socket.timeout):
+            with tests.assert_raises(dns.exception.Timeout):
+                greendns.udp(self.query, '::1', timeout=0.1, ignore_unexpected=True)
+
+    def test_udp_ipv6_wrong_addr(self):
+        with tests.mock.patch('eventlet.support.greendns.socket.socket.recvfrom',
+                              return_value=(self.query_wire,
+                                            ('ffff:0000::1', 53, 0, 0))):
+            with tests.assert_raises(dns.query.UnexpectedSource):
+                greendns.udp(self.query, '::1')
+
+
 class TestProxyResolver(tests.LimitedTestCase):
 
     def test_clear(self):
@@ -319,7 +396,7 @@ class TestResolve(tests.LimitedTestCase):
         assert greendns.resolver.args == ('host.example.com', dns.rdatatype.A)
 
     def test_AAAA(self):
-        greendns.resolver.rr.address = 'dead:beef::1'
+        greendns.resolver.rr6.address = 'dead:beef::1'
         ans = greendns.resolve('host.example.com', socket.AF_INET6)
         assert ans[0].address == 'dead:beef::1'
         assert greendns.resolver.args == ('host.example.com', dns.rdatatype.AAAA)
@@ -394,7 +471,8 @@ def _make_mock_resolve():
         def __init__(self):
             self.answers = {}
 
-        def __call__(self, name, family=socket.AF_INET, raises=True):
+        def __call__(self, name, family=socket.AF_INET, raises=True,
+                     _proxy=None, use_network=True):
             qname = dns.name.from_text(name)
             try:
                 rrset = self.answers[name][family]
@@ -776,6 +854,31 @@ class TestGethostbyname_ex(tests.LimitedTestCase):
         assert res == ('host.example.com', [], ['1.2.3.4', '1.2.3.5'])
 
 
+class TinyDNSTests(tests.LimitedTestCase):
+
+    def test_raise_dns_tcp(self):
+        # https://github.com/eventlet/eventlet/issues/499
+        # None means we don't want the server to find the IP
+        with tests.dns_tcp_server(None) as dnsaddr:
+            resolver = Resolver()
+            resolver.nameservers = [dnsaddr[0]]
+            resolver.nameserver_ports[dnsaddr[0]] = dnsaddr[1]
+
+            with self.assertRaises(NoAnswer):
+                resolver.query('host.example.com', 'a', tcp=True)
+
+    def test_noraise_dns_tcp(self):
+        # https://github.com/eventlet/eventlet/issues/499
+        expected_ip = "192.168.1.1"
+        with tests.dns_tcp_server(expected_ip) as dnsaddr:
+            resolver = Resolver()
+            resolver.nameservers = [dnsaddr[0]]
+            resolver.nameserver_ports[dnsaddr[0]] = dnsaddr[1]
+            response = resolver.query('host.example.com', 'a', tcp=True)
+            self.assertIsInstance(response, Answer)
+            self.assertEqual(response.rrset.items[0].address, expected_ip)
+
+
 def test_reverse_name():
     tests.run_isolated('greendns_from_address_203.py')
 
@@ -818,3 +921,60 @@ def test_hosts_priority():
     rrs6 = greendns.resolve(name, family=socket.AF_INET6, _proxy=rp).rrset
     assert len(rrs6) == 1
     assert rrs6[0].address == 'dead:beef::1', rrs6[0].address
+
+
+def test_hosts_no_network():
+
+    name = 'example.com'
+    addr_from_ns = '1.0.2.0'
+    addr6_from_ns = 'dead:beef::1'
+
+    hr = _make_host_resolver()
+    rp = greendns.ResolverProxy(hosts_resolver=hr, filename=None)
+    base = _make_mock_base_resolver()
+    base.rr.address = addr_from_ns
+    base.rr6.address = addr6_from_ns
+    rp._resolver = base()
+
+    with tests.mock.patch.object(greendns, 'resolver',
+                                 new_callable=tests.mock.PropertyMock(return_value=rp)):
+        res = greendns.getaddrinfo('example.com', 'domain', socket.AF_UNSPEC)
+        # Default behavior
+        addr = (addr_from_ns, 53)
+        tcp = (socket.AF_INET, socket.SOCK_STREAM, socket.IPPROTO_TCP, addr)
+        udp = (socket.AF_INET, socket.SOCK_DGRAM, socket.IPPROTO_UDP, addr)
+        addr = (addr6_from_ns, 53, 0, 0)
+        tcp6 = (socket.AF_INET6, socket.SOCK_STREAM, socket.IPPROTO_TCP, addr)
+        udp6 = (socket.AF_INET6, socket.SOCK_DGRAM, socket.IPPROTO_UDP, addr)
+        filt_res = [ai[:3] + (ai[4],) for ai in res]
+        assert tcp in filt_res
+        assert udp in filt_res
+        assert tcp6 in filt_res
+        assert udp6 in filt_res
+
+        # Hosts result must shadow that from nameservers
+        hr = _make_host_resolver()
+        hr.hosts.write(b'1.2.3.4 example.com')
+        hr.hosts.flush()
+        hr._load()
+        greendns.resolver._hosts = hr
+
+        res = greendns.getaddrinfo('example.com', 'domain', socket.AF_UNSPEC)
+        filt_res = [ai[:3] + (ai[4],) for ai in res]
+
+        # Make sure that only IPv4 entry from hosts is present.
+        assert tcp not in filt_res
+        assert udp not in filt_res
+        assert tcp6 not in filt_res
+        assert udp6 not in filt_res
+
+        addr = ('1.2.3.4', 53)
+        tcp = (socket.AF_INET, socket.SOCK_STREAM, socket.IPPROTO_TCP, addr)
+        udp = (socket.AF_INET, socket.SOCK_DGRAM, socket.IPPROTO_UDP, addr)
+        assert tcp in filt_res
+        assert udp in filt_res
+
+
+def test_import_rdtypes_then_eventlet():
+    # https://github.com/eventlet/eventlet/issues/479
+    tests.run_isolated('greendns_import_rdtypes_then_eventlet.py')
